#ifndef _BOX_H_
#define _BOX_H_

#include "chamber.h"
#include "stats.h"
#include <fstream>

using namespace std;

class Box
    : public Chamber
{
public:
    ~Box();
    Box(const char* filename);
//    Box(istream is);

// inherited Chamber interface
    double radius() const;// biggest lateral dimension, maybe replace by (x,y)?
    double gravity() const;

    void useSphereDiameter(double diam);

    double level(double volume) const;

    int isInside(const double *r) const;
    int isOutside(const double *r) const;

    void advance(double time);
    double detect(const double *r, const double *v, double best_time);
    void collide(const double *r, double *v);

    void reset();

protected:
    double coef_sb(double velocity) const;
    double coef_sw(double velocity) const;
    double coef_st(double velocity) const;

    void dumpImpact2D(const double* r, const char* filename) const;
    void dumpImpact2D(const double* r, ofstream&) const;

//DATA
    double xsize0_, ysize0_;// actual box dimensions(halved)
    double xsize_, ysize_;// same reduced by particle radius
    double height0_, height_;
    double gravity_, gravity_half_, gravity_inv_;

// collision model
    double
        coef_sb_, v_scale_sb_,
        coef_sw_, v_scale_sw_,
        coef_st_, v_scale_st_,
        r_tol_;// tolerance for particle VS wall tests

// sawtooth motion of the base : constants
    double base_amplitude_;
    double base_period_, base_period_half_, base_period_inv_;
    double base_velocity_, base_velocity_inv_;

// sawtooth motion of the base : variables
    double base_z_;// position is centered at 0
    int base_dir_;

// collision data
    int col_type_;

// collision counters
    int col_st_, col_sb_, col_swx_, col_swy_;

// impact statistics
    char impact_map_top_file_[30];
    char impact_map_bot_file_[30];
    ofstream impact_map_top_;
    ofstream impact_map_bot_;

    int maps2D_;// toggles dumping of every impact's coordinates to file
    int xcells_, ycells_;// cells for impact histograms
    Stats2Dxy top_hist_, bot_hist_;// impact histograms
};

inline double Box::radius() const {return sqrt(xsize_*xsize_ + ysize_*ysize_);}
inline double Box::gravity() const {return gravity_;}

#endif
