#include "boxFGL.h"
#include <GL/freeglut.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>

using namespace std;

static void
drawBox(
    GLfloat x0, GLfloat x1,
    GLfloat y0, GLfloat y1,
    GLfloat z0, GLfloat z1,
    GLenum type)
{
    static GLfloat n[6][3] =
    {
	{-1.0, 0.0, 0.0},
	{0.0, 1.0, 0.0},
	{1.0, 0.0, 0.0},
	{0.0, -1.0, 0.0},
	{0.0, 0.0, 1.0},
	{0.0, 0.0, -1.0}
    };
    static GLint faces[6][4] =
    {
	{0, 1, 2, 3},
	{3, 2, 6, 7},
	{7, 6, 5, 4},
	{4, 5, 1, 0},
	{5, 6, 2, 1},
	{7, 4, 0, 3}
    };
    GLfloat v[8][3], tmp;
    GLint i;

    if (x0 > x1) {
	tmp = x0;
	x0 = x1;
	x1 = tmp;
    }
    if (y0 > y1) {
	tmp = y0;
	y0 = y1;
	y1 = tmp;
    }
    if (z0 > z1) {
	tmp = z0;
	z0 = z1;
	z1 = tmp;
    }
    v[0][0] = v[1][0] = v[2][0] = v[3][0] = x0;
    v[4][0] = v[5][0] = v[6][0] = v[7][0] = x1;
    v[0][1] = v[1][1] = v[4][1] = v[5][1] = y0;
    v[2][1] = v[3][1] = v[6][1] = v[7][1] = y1;
    v[0][2] = v[3][2] = v[4][2] = v[7][2] = z0;
    v[1][2] = v[2][2] = v[5][2] = v[6][2] = z1;

    for (i = 0; i < 6; i++) {
	glBegin(type);
	glNormal3fv(&n[i][0]);
	glVertex3fv(&v[faces[i][0]][0]);
	glVertex3fv(&v[faces[i][1]][0]);
	glVertex3fv(&v[faces[i][2]][0]);
	glVertex3fv(&v[faces[i][3]][0]);
	glEnd();
    }
}

static void
glutWireBox(
    GLdouble x0, GLdouble x1,
    GLdouble y0, GLdouble y1,
    GLdouble z0, GLdouble z1)
{
    drawBox(x0, x1, y0, y1, z0, z1, GL_LINE_LOOP);
}

static void
glutSolidBox(
    GLdouble x0, GLdouble x1,
    GLdouble y0, GLdouble y1,
    GLdouble z0, GLdouble z1)
{
    drawBox(x0, x1, y0, y1, z0, z1, GL_QUADS);
}

/*static void glutSolidBox(double dx, double dy, double height)
{
    glPushMatrix();
    glBegin(GL_QUADS);

    glNormal3f(0., 0., 1.);// TOP
    glVertex3f(-dx,-dy, height);
    glVertex3f(-dx, dy, height);
    glVertex3f( dx, dy, height);
    glVertex3f( dx,-dy, height);

    glNormal3f(0.0F,0.0F,-1.0F);// BOTTOM
    glVertex3f(-dx,-dy, 0.);
    glVertex3f(-dx, dy, 0.);
    glVertex3f( dx, dy, 0.);
    glVertex3f( dx,-dy, 0.);

    glNormal3f(0.0F,1.0F,0.0F);// LEFT AND RIGHT
    glVertex3f(-dx, dy, 0.);
    glVertex3f(-dx, dy, height);
    glVertex3f( dx, dy, height);
    glVertex3f( dx, dy, 0.);
    glNormal3f(0.0F,-1.0F,0.0F);
    glVertex3f(-dx,-dy, 0.);
    glVertex3f(-dx,-dy, height);
    glVertex3f( dx,-dy, height);
    glVertex3f( dx,-dy, 0.);

    glNormal3f(1.0F,0.0F,0.0F);// FRONT AND BACK
    glVertex3f(dx,yi,zi);
    glVertex3f(dx/2,yi,zf);
    glVertex3f(dx/2,yf,zf);
    glVertex3f(dx/2,yf,zi);
    glNormal3f(-1.0F,0.0F,0.0F);
    glVertex3f(-dx/2,yi,zi);
    glVertex3f(-dx/2,yi,zf);
    glVertex3f(-dx/2,yf,zf);
    glVertex3f(-dx/2,yf,zi);

    glEnd();
    glPopMatrix();
}
*/

BoxFGL::~BoxFGL()
{
}

BoxFGL::BoxFGL(const char* ini, const char* iniGL)
    : Box(ini)
{
    ifstream is(iniGL);
    if(!is.is_open()) {cerr <<"Error reading "<< iniGL << endl; exit(1);}

    is >> GLheight_; while(is.get()!='\n'); if(height_>0) GLheight_= height0_;

    is >> GLbaseR_ >> GLbaseG_ >> GLbaseB_ >> GLbaseA_; while(is.get()!='\n');
    is >> GLwallR_ >> GLwallG_ >> GLwallB_ >> GLwallA_; while(is.get()!='\n');

    if(!is) {cerr <<"Error reading "<< iniGL << endl; exit(1);}
}

void
BoxFGL::draw() const
{
    double thickness= .1;

    glEnable(GL_BLEND);
    glDisable(GL_DEPTH_TEST);

// DRAW BOX WALL (transparent)
    glColor4f(GLwallR_, GLwallG_, GLwallB_, GLwallA_);
    glutSolidBox(-xsize0_, xsize0_, -ysize0_, ysize0_, base_z_-thickness*.5, height0_);

    glDisable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);

    glColor4f(GLbaseR_, GLbaseG_, GLbaseB_, GLbaseA_);

// DRAW RIM
    glLineWidth(5.);
    glutWireBox(-xsize0_, xsize0_, -ysize0_, ysize0_, base_z_-thickness*.5, height0_);
    glLineWidth(1.);

// DRAW MOVING BASE
    glutSolidBox(-xsize0_, xsize0_, -ysize0_, ysize0_, base_z_-thickness, base_z_);
}

void
BoxFGL::console()
{
}

