#include <cfloat>
#include <iostream>
#include <fstream>
#include "cylinder.h"
#include "echo.h"
#undef max
#define max(a,b) ((a) > (b) ? (a) : (b))
#undef min
#define min(a,b) ((a) < (b) ? (a) : (b))

using namespace std;

enum {SS, SB, SW, ST}; // collision type : chamber-specific

double
Cylinder::detect(const double *r, const double *v, double best_time, double gravity)
{
    double t, delta;
// detect collision with cylinder top if any
    if (height_) {
        t= DBL_MAX;
        double vz2= v[Z]*v[Z];
        delta= vz2 - 2.*(height_ - r[Z])*gravity;
        if (v[Z]>0 && delta > 0) t= (v[Z] - sqrt(delta))/gravity; // division !!!!!!!
        if(t < best_time){
            best_time= t;
            col_type_= ST;
//cout << "ST detected "<< t<< endl;
        }
    }
// detect collision with cylinder wall
    double b, r2D2, v2D2, dr2D2;

    b= v[X]*r[X] + v[Y]*r[Y];
    r2D2= r[X]*r[X] + r[Y]*r[Y];
    v2D2= v[X]*v[X] + v[Y]*v[Y];
    if (r2D2 >(radius2_ + 2e-09)) // particle is outside the cylinder wall
        cout <<"particle at (x,y)=("<<r[X]<<","<<r[Y]<<") "
            <<"with velocity (vx,vy)=("<<v[X]<<","<<v[Y]<<")"<< endl
            <<"has (x^2+y^2="<<r2D2<<")>(R^2="<<radius2_<<") "
            <<"i.e. is outside the cylinder wall"<<endl;

    dr2D2= max(radius2_ - r2D2, 0.);
    t= (-b + sqrt(dr2D2*v2D2 + b*b))/v2D2; // division!!!!!!
    if (t < best_time) {
        best_time= t;
        col_type_= SW;
    }

// DETECT COLLISION WITH CYLINDER BASE
    int flag;
    double d_1, vv;
    double tmin, tmax, rzmin, vzmin, zwmin;
    double zi,zp,zt0,vt0,rzt0,t0;

    t0= (base_amplitude_ - base_dir_ * base_z_)/base_velocity_;
    zt0= base_z_ + base_dir_ *base_velocity_*t0;
    vt0= base_dir_ * base_velocity_;
    flag= 0;
    if(r[Z] < base_z_ - 1e-3)
        cout <<"sphere below wall : (z_i="<<r[Z]<<")<(z_wall="<<base_z_<<")"<< endl;
/* on calcule d'abord le tmin et le tmax de chaque particule */

    if(r[Z] > base_amplitude_){
        tmin=(v[Z] + sqrt(v[Z]*v[Z] + 2.*(r[Z] - base_amplitude_)*gravity))/gravity;
/*
        rzmin= r[Z] + tmin*(v[Z] - gravity *0.5*tmin);
*/
        rzmin= base_amplitude_;
        vzmin= v[Z] - gravity *tmin;
    }
    else{tmin=0; rzmin=r[Z]; vzmin=v[Z];}

    tmax=(v[Z] + sqrt(v[Z]*v[Z] + 2*(r[Z] + base_amplitude_)*gravity))/gravity;

/* on ne fait les tests suivants que si la borne infrieure
est infrieure  tnext */
    if(tmin < best_time){
        while(tmin > t0){
            t0+= base_period_half_;
            zt0= -zt0;
            vt0= -vt0;
        }
        zwmin= zt0 + vt0*(tmin-t0);
        if(tmax<t0){
            flag=1;
        }
        else{/*tmax suprieur  t0*/
            rzt0= r[Z] + t0*(v[Z] - gravity*0.5*t0);
            if(rzt0 < zt0) flag= 1;
            else{
                do{
                    tmin= t0;
                    rzmin= r[Z] + tmin*(v[Z] - gravity*0.5*tmin);
                    vzmin= v[Z] - gravity*tmin;
                    t0+= base_period_half_;
                    zt0= -zt0;
                    vt0= -vt0;
                    rzt0= r[Z] + t0*(v[Z] - gravity*0.5*t0);
                }while(rzt0 > zt0);
                flag=1;
            }
        }
        if(flag ==1){
            d_1= vzmin - vt0;
            vv= d_1*d_1 + gravity*2.0*(rzmin - zwmin);
            t= (d_1 + sqrt(vv))/gravity;
            zi= zwmin + vt0*t;
            zp= rzmin + t*(vzmin - gravity*0.5*t);
            if(fabs(zi - zp) > 1e-6){
                cout <<"base problem "; echo3(zi,zp,t);
                exit(1);
            }
            if((t + tmin) < best_time){
                best_time= t + tmin;
                col_type_= SB;
            }
        }
    }
    return best_time;
}
