#include "experiment.h"
#include "shot.h"
#include "chamber.h"
#include "par.h"
#include "timer.h"
#include <iostream>
#include <fstream>

using namespace std;

Experiment::~Experiment() {}

Experiment::Experiment(Shot& shot, Chamber& chamber, const Par& par)
    : shot_(shot)
    , chamber_(chamber)
    , ncolmax_(par.ncolmax())
    , events_(0)
    , restart_file_(par.restartFile())
    , tstep_(par.tStep())
    , tmax_(par.tMax())
{
    reset();
}

void
Experiment::makeStep(double tstep)
{
    while(tnext_ < tstep) {
// GO TO DETECTED COLLISION
        tstep-= tnext_; tfull_+= tnext_;
        chamber_.advance(tnext_);
        shot_.advance(chamber_.gravity());

//        shot_.test(chamber_);

// PERFORM COLLISION
        shot_.collide(chamber_); ++events_;

// DETECT NEXT COLLISION
        shot_.detect();// detect/update intrashot collisions
        shot_.detect(chamber_);// detect imminent chamber impact
        tnext_= shot_.tNext();
    }
//cout<<shot_.tNext()<<endl;
    chamber_.advance(tstep);
    shot_.advance(chamber_.gravity(), tstep);
    tnext_-= tstep; tfull_+= tstep;
}

void
Experiment::fullRun()
{
    cout << "Computing "<< ncolmax_ <<" events / "<< tmax_ <<" cycles..."<<endl;
    Timer timer("cpu");
    while(events_<ncolmax_ || tfull_<tmax_) makeStep(tstep_);
    cout << events_ <<" events computed in "<< timer <<" seconds"<< endl;
}

void
Experiment::reset()
{
    tfull_= 0.;
    events_= 0;
    chamber_.reset();
    shot_.set(chamber_);
    if(restart_file_) {
        ifstream is(restart_file_);
        if(!is.is_open()) {
            cerr << "Error reading " << restart_file_ << endl;
            exit(1);
        }
        shot_.readState(is);
        if(!is) {
            cerr << "Error reading " << restart_file_ << endl;
            exit(1);
        }
    }
    else shot_.distribute(chamber_);
    shot_.test(chamber_);
//    shot_.dumpState(cout);
    shot_.detect();
    shot_.detect(chamber_);
    tnext_= shot_.tNext();
}

