#include "shot.h"
#include "chamber.h"

#include "cylinderFGL.h"

#include "interfaceFGL.h"
#include "experimentFGL.h"
#include <GL/freeglut.h>
#include <iostream>
#include <fstream>

using namespace std;

static void
textString(int x, int y, char *msg, void *font)
{
    glRasterPos2f(x, y);
    while (*msg) {
        glutBitmapCharacter(font, *msg);
        msg++;
    }
}

ExperimentFGL::~ExperimentFGL()
{
}

ExperimentFGL::ExperimentFGL(Shot& shot, Chamber& chamber, const Par& par, const char* iniFGL)
    : Experiment(shot, chamber, par)
    , GLb_left_(0)
    , GLb_right_(0)
    , GLspeed_(10)
    , GLtstep_(tstep_*GLspeed_*.1)
{
    ifstream is(iniFGL);
    if(!is.is_open()) {cerr <<"Error reading "<< iniFGL << endl; exit(1);}

    is >> GLdistance_min_ >> GLdistance_max_; while(is.get()!='\n');
    GLdistance_= GLdistance_max_;
    is >> GLztarget_; while(is.get()!='\n');
    is >> GLphi_ >> GLtheta_; while(is.get()!='\n');
    if(!is) {cerr <<"Error reading "<< iniFGL << endl; exit(1);}
}

void
ExperimentFGL::initFGL(int argc, char** argv) const
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH | GLUT_ALPHA);
    glEnable (GL_BLEND);
    glutInitWindowSize(500, 704);
    glutInitWindowPosition(520, 0);
    glutCreateWindow("Real-time Animation v1.0");

    glClearColor(.8, .8, .8, 1.);
    glColor3ui(0, 0, 0);
    glPointSize(3.5);

    glEnable(GL_DEPTH_TEST);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45., 1., .1, 200.);
    glMatrixMode(GL_MODELVIEW);

    glEnable(GL_NORMALIZE);
    glEnable(GL_COLOR_MATERIAL);
    glEnable(GL_LIGHTING);
    glEnable(GL_ALPHA_TEST);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

const GLfloat light_position[] = { 2.0f, 5.0f, 5.0f, 0.0f };
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);
    glEnable(GL_LIGHT0);
}

void
ExperimentFGL::idleCallback()
{
//    int cols= events_;
    makeStep(GLtstep_);
    glutPostRedisplay();
//    if(events_>cols) {
//        GLspeed_= 0;
//        GLtstep_= 0.;
//    }
}

void
ExperimentFGL::displayCallback()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();
    gluLookAt(GLdistance_, 0., 0., 0., 0., GLztarget_, 0., 0., 1.);
    glRotatef(GLtheta_, 0., 1., 0.);
    glRotatef(GLphi_, 0., 0., 1.);
    glColor3f(1., 1.,1.);

    dynamic_cast<InterfaceFGL&>(shot_).draw();
    dynamic_cast<InterfaceFGL&>(chamber_).draw();

    glDisable(GL_LIGHTING);
    showText();
    glEnable(GL_LIGHTING);
    glutSwapBuffers();
}

void
ExperimentFGL::mouseCallback(int button, int state, int x, int y)
{
    if(button==GLUT_LEFT_BUTTON) {
        if(state==GLUT_DOWN) {
            GLb_left_= 1;
            GLx_prev_= x;
            GLy_prev_= y;
        }
        else GLb_left_= 0;
    }
    else if(button==GLUT_RIGHT_BUTTON) {
        if(state==GLUT_DOWN) {
            GLb_right_= 1;
            GLy_prev_= y;
        }
        else GLb_right_= 0;
    }
}

void
ExperimentFGL::keyboardCallback(unsigned char key, int x, int y)
{
    switch(key) {
        case 32:// Space
            reset();
            break;

        case 27:// ESC
            exit(0);
            break;
    }
}

void
ExperimentFGL::specialCallback(int key, int x, int y)
{
    switch (key) {
        case GLUT_KEY_UP:
            ++GLspeed_;
            GLtstep_= .1*GLspeed_*tstep_;
            break;
        case GLUT_KEY_DOWN:
            --GLspeed_;
            if(GLspeed_ < 0) GLspeed_ = 0;
            GLtstep_= .1*GLspeed_*tstep_;
            break;
    }
}

void
ExperimentFGL::motionCallback(int x, int y)
{
    if (GLb_left_) {
        GLphi_+= x - GLx_prev_;
        if (GLphi_ >= 360) GLphi_ %= 360;
        if (GLphi_ < 0) while (GLphi_ < 0) GLphi_ += 360;
        GLtheta_+= y - GLy_prev_;
        if (GLtheta_ >= 360) GLtheta_ %= 360;
        if (GLtheta_ < 0) while (GLtheta_ < 0) GLtheta_ += 360;
        GLx_prev_= x; GLy_prev_= y;
    }
    if (GLb_right_) {
        GLdistance_+= y - GLy_prev_;
        if (GLdistance_ < GLdistance_min_) GLdistance_ = GLdistance_min_;
        else if(GLdistance_ > GLdistance_max_) GLdistance_ = GLdistance_max_;
        GLy_prev_= y;
    }
}

void
ExperimentFGL::reshapeCallback(int width, int height)
{
    const float ar = (float) width / (float) height;

    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustum(-ar, ar, -1.0, 1.0, 2.0, 100.0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity() ;
}

void
ExperimentFGL::showText()
{
    char particles[30];
    char speed[30];
    char collision[30];
    char gravity[30];
    char time[30];

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    gluOrtho2D(0, 50, 0, 50);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();

    glColor3f(0., 0., 0.);
//    glIndexi(0);


    glDisable(GL_DEPTH_TEST);

    sprintf(particles, "%ld particles", shot_.n());
    sprintf(speed, "speed %ld", GLspeed_);
    sprintf(collision, "%ld events", events_);
    sprintf(gravity, "g = %g", chamber_.gravity());
    sprintf(time, "%ld cycles", int(tfull_));

    textString(1, 48, collision, GLUT_BITMAP_TIMES_ROMAN_24);
    textString(22, 48, speed, GLUT_BITMAP_TIMES_ROMAN_24);
    textString(36, 48, particles, GLUT_BITMAP_TIMES_ROMAN_24);
    textString(1, 1, gravity, GLUT_BITMAP_TIMES_ROMAN_24);
    textString(22, 1, time, GLUT_BITMAP_TIMES_ROMAN_24);

    glEnable(GL_DEPTH_TEST);

    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
}

