#ifndef _SHOT_H_
#define _SHOT_H_

#include <iostream>
//#include <fstream>

using namespace std;

class Chamber;
class Stats3D;

// monodisperse set of hard spheres in an homogeneous gravitational field
class Shot
{
public:
    virtual ~Shot();
    Shot(const char* filename);

// accessors (some are only useful for GL...)
    int n() const;
    const double* r_chunk() const;
    const double* v_chunk() const;
    int collision() const;
    double diam() const;
    double tNext() const;

// actions
    void set(Chamber&) const;//tell Chamber about particle radius

    long distribute(Chamber&);// distribute spheres randomly in chamber

    void detect();// detect/update intrashot collisions
    void detect(Chamber&);// detect chamber impact

    void advance(double gravity, double time);// advance to generic time
    void advance(double gravity) {advance(gravity, best_time_);}

    void collide(Chamber&);// perform collision/impact

// auxiliary stuff

    void test(const Chamber&) const;// overlap/out-of-chamber test

    void readState(istream&);
    void dumpState(ostream&) const;
//    void dumpState(Stats3D&) const;

//    void makeStatEntry(Chamber&);

protected:
    void resetCollisionData();

    double coef(double velocity2) const;// restitution coefficient

// shot properties
    int n_;
    double diam_, diam2_, radius_;// diameter, diameter squared, radius

// starting configuration
    double
        eta_,// packing fraction of random distribution
        kt_,// kt/m : temperature of random distribution
        omega0_;// solid rotation speed

    double rtol_, r2tol_;

// collision model
    double
        coef0_,// elastic restitution coefficient
        v2_scale_;// inverse of threshold velocity squared

    double **r_, **v_;// state vector

// collision data
    double *tnext_;// time to next collision for each particle
    int *partner_;// its collision partner
    int *update_;// collision update flag
    int chosen_;// chosen particle
    double best_time_;// best time yet (including chamber impacts)
    int chamber_impact_;// 0 for intrashot impacts, 1 otherwise

    int collision_;// intrashot collision counter
};

inline int Shot::n() const {return n_;}
inline const double* Shot::r_chunk() const {return r_[0];}
inline const double* Shot::v_chunk() const {return v_[0];}
inline int Shot::collision() const {return collision_;}
inline double Shot::diam() const {return diam_;}
inline double Shot::tNext() const {return best_time_;}

#endif

